/****************************************************************************
**
** Copyright (C) 2015 The Qt Company Ltd.
** Copyright (C) 2016 Ivailo Monev
**
** This file is part of the QtCore module of the Katie Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
**
** GNU Lesser General Public License Usage
** This file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef QLOCALE_H
#define QLOCALE_H

#include <QtCore/qvariant.h>


QT_BEGIN_NAMESPACE

class QDataStream;
class QDate;
class QDateTime;
class QTime;

class QLocalePrivate;

class Q_CORE_EXPORT QLocale
{
    Q_GADGET
    Q_ENUMS(Language)
    Q_ENUMS(Country)
    friend class QString;
    friend class QByteArray;
    friend class QIntValidator;
    friend class QDoubleValidatorPrivate;
    friend class QTextStream;
    friend class QTextStreamPrivate;

public:
    // GENERATED PART STARTS HERE
    // see genlocale.py for more info on generated data
    enum Language {
        AnyLanguage = 0,
        C = 1,
        Abkhazian = 2,
        Achinese = 3,
        Acoli = 4,
        Adangme = 5,
        Adyghe = 6,
        Afar = 7,
        Afrihili = 8,
        Afrikaans = 9,
        Aghem = 10,
        Ainu = 11,
        Akan = 12,
        Akkadian = 13,
        Akoose = 14,
        Alabama = 15,
        Albanian = 16,
        Aleut = 17,
        AlgerianArabic = 18,
        AmericanEnglish = 19,
        AmericanSignLanguage = 20,
        Amharic = 21,
        AncientEgyptian = 22,
        AncientGreek = 23,
        Angika = 24,
        AoNaga = 25,
        Arabic = 26,
        ArabicNajdi = 27,
        Aragonese = 28,
        Aramaic = 29,
        Araona = 30,
        Arapaho = 31,
        Arawak = 32,
        Armenian = 33,
        Aromanian = 34,
        Arpitan = 35,
        Assamese = 36,
        Asturian = 37,
        Asu = 38,
        Atikamekw = 39,
        Atsam = 40,
        AustralianEnglish = 41,
        AustrianGerman = 42,
        Avaric = 43,
        Avestan = 44,
        Awadhi = 45,
        Aymara = 46,
        Azerbaijani = 47,
        Badaga = 48,
        Bafia = 49,
        Bafut = 50,
        Bakhtiari = 51,
        Balinese = 52,
        Baluchi = 53,
        Bambara = 54,
        Bamun = 55,
        Bangla = 56,
        Banjar = 57,
        Basaa = 58,
        Bashkir = 59,
        Basque = 60,
        BatakToba = 61,
        Bavarian = 62,
        Beja = 63,
        Belarusian = 64,
        Bemba = 65,
        Bena = 66,
        Betawi = 67,
        Bhojpuri = 68,
        Bikol = 69,
        Bini = 70,
        Bishnupriya = 71,
        Bislama = 72,
        Blin = 73,
        Blissymbols = 74,
        Bodo = 75,
        Bosnian = 76,
        Brahui = 77,
        Braj = 78,
        BrazilianPortuguese = 79,
        Breton = 80,
        BritishEnglish = 81,
        Buginese = 82,
        Bulgarian = 83,
        Bulu = 84,
        Buriat = 85,
        Burmese = 86,
        Caddo = 87,
        CajunFrench = 88,
        CanadianEnglish = 89,
        CanadianFrench = 90,
        Cantonese = 91,
        Capiznon = 92,
        Carib = 93,
        CarolinaAlgonquian = 94,
        Catalan = 95,
        Cayuga = 96,
        Cebuano = 97,
        CentralAtlasTamazight = 98,
        CentralDusun = 99,
        CentralKurdish = 100,
        CentralOjibwa = 101,
        CentralYupik = 102,
        ChadianArabic = 103,
        Chagatai = 104,
        Chakma = 105,
        Chamorro = 106,
        Chechen = 107,
        Cherokee = 108,
        Cheyenne = 109,
        Chibcha = 110,
        Chickasaw = 111,
        Chiga = 112,
        Chilcotin = 113,
        ChimborazoHighlandQuichua = 114,
        Chinese = 115,
        ChinookJargon = 116,
        Chipewyan = 117,
        Choctaw = 118,
        ChurchSlavic = 119,
        Chuukese = 120,
        Chuvash = 121,
        ClassicalNewari = 122,
        ClassicalSyriac = 123,
        Colognian = 124,
        Comorian = 125,
        CongoSwahili = 126,
        Coptic = 127,
        Cornish = 128,
        Corsican = 129,
        Cree = 130,
        CrimeanTatar = 131,
        Croatian = 132,
        Czech = 133,
        Dakota = 134,
        Danish = 135,
        Dargwa = 136,
        Dari = 137,
        Dazaga = 138,
        Delaware = 139,
        Dinka = 140,
        Divehi = 141,
        Dogri = 142,
        Dogrib = 143,
        Duala = 144,
        Dutch = 145,
        Dyula = 146,
        Dzongkha = 147,
        EasternCanadianInuktitut = 148,
        EasternFrisian = 149,
        EasternOjibwa = 150,
        Efik = 151,
        EgyptianArabic = 152,
        Ekajuk = 153,
        Elamite = 154,
        Embu = 155,
        Emilian = 156,
        English = 157,
        Erzya = 158,
        Esperanto = 159,
        Estonian = 160,
        EuropeanPortuguese = 161,
        EuropeanSpanish = 162,
        Ewe = 163,
        Ewondo = 164,
        Extremaduran = 165,
        Fang = 166,
        Fanti = 167,
        Faroese = 168,
        FijiHindi = 169,
        Fijian = 170,
        Filipino = 171,
        Finnish = 172,
        Flemish = 173,
        Fon = 174,
        Frafra = 175,
        French = 176,
        Friulian = 177,
        Fula = 178,
        Ga = 179,
        Gagauz = 180,
        Galician = 181,
        GanChinese = 182,
        Ganda = 183,
        Gayo = 184,
        Gbaya = 185,
        Geez = 186,
        Georgian = 187,
        German = 188,
        GhegAlbanian = 189,
        Ghomala = 190,
        Gilaki = 191,
        Gilbertese = 192,
        GoanKonkani = 193,
        Gondi = 194,
        Gorontalo = 195,
        Gothic = 196,
        Grebo = 197,
        Greek = 198,
        Guarani = 199,
        Gujarati = 200,
        Gusii = 201,
        Gwichin = 202,
        Haida = 203,
        HaitianCreole = 204,
        HakkaChinese = 205,
        Halkomelem = 206,
        Haryanvi = 207,
        Hausa = 208,
        Hawaiian = 209,
        Hebrew = 210,
        Herero = 211,
        Hiligaynon = 212,
        Hindi = 213,
        HindiLatin = 214,
        HiriMotu = 215,
        Hittite = 216,
        Hmong = 217,
        HmongNjua = 218,
        Hungarian = 219,
        Hupa = 220,
        Iban = 221,
        Ibibio = 222,
        Icelandic = 223,
        Ido = 224,
        Igbo = 225,
        Iloko = 226,
        InariSami = 227,
        Indonesian = 228,
        Ingrian = 229,
        Ingush = 230,
        Innuaimun = 231,
        Interlingua = 232,
        Interlingue = 233,
        Inuktitut = 234,
        Inupiaq = 235,
        Irish = 236,
        Italian = 237,
        JamaicanCreoleEnglish = 238,
        Japanese = 239,
        Javanese = 240,
        Jju = 241,
        JolaFonyi = 242,
        JudeoArabic = 243,
        JudeoPersian = 244,
        Jutish = 245,
        Kabardian = 246,
        Kabuverdianu = 247,
        Kabyle = 248,
        Kachin = 249,
        Kaingang = 250,
        Kako = 251,
        Kalaallisut = 252,
        Kalenjin = 253,
        Kalmyk = 254,
        Kamba = 255,
        Kanembu = 256,
        Kannada = 257,
        Kanuri = 258,
        KaraKalpak = 259,
        KarachayBalkar = 260,
        Karelian = 261,
        Kashmiri = 262,
        Kashubian = 263,
        Kawi = 264,
        Kazakh = 265,
        Kenyang = 266,
        Khasi = 267,
        Khmer = 268,
        Khotanese = 269,
        Khowar = 270,
        Kiche = 271,
        Kikuyu = 272,
        Kimbundu = 273,
        Kinaraya = 274,
        Kinyarwanda = 275,
        Kirghiz = 276,
        Kirmanjki = 277,
        Klingon = 278,
        Kom = 279,
        Komi = 280,
        KomiPermyak = 281,
        Kongo = 282,
        Konkani = 283,
        Korean = 284,
        Koro = 285,
        Kosraean = 286,
        Kotava = 287,
        KoyraChiini = 288,
        KoyraboroSenni = 289,
        Kpelle = 290,
        Krio = 291,
        Kuanyama = 292,
        Kumyk = 293,
        Kurdish = 294,
        Kurukh = 295,
        Kutenai = 296,
        Kwakwala = 297,
        Kwasio = 298,
        Ladino = 299,
        Lakota = 300,
        Lamba = 301,
        Langi = 302,
        Lao = 303,
        Latgalian = 304,
        Latin = 305,
        LatinAmericanSpanish = 306,
        Latvian = 307,
        Laz = 308,
        Lezghian = 309,
        Ligurian = 310,
        Lillooet = 311,
        Limburgish = 312,
        Lingala = 313,
        LinguaFrancaNova = 314,
        LiteraryChinese = 315,
        Lithuanian = 316,
        Livonian = 317,
        Lojban = 318,
        Lombard = 319,
        LouisianaCreole = 320,
        LowGerman = 321,
        LowSaxon = 322,
        LowerSilesian = 323,
        LowerSorbian = 324,
        Lozi = 325,
        LubaKatanga = 326,
        LubaLulua = 327,
        Luiseno = 328,
        LuleSami = 329,
        Lunda = 330,
        Luo = 331,
        Luxembourgish = 332,
        Luyia = 333,
        Maba = 334,
        Macedonian = 335,
        Machame = 336,
        Madurese = 337,
        Mafa = 338,
        Magahi = 339,
        MainFranconian = 340,
        Maithili = 341,
        Makasar = 342,
        MakhuwaMeetto = 343,
        Makonde = 344,
        Malagasy = 345,
        Malay = 346,
        Malayalam = 347,
        MaliseetPassamaquoddy = 348,
        Maltese = 349,
        Manchu = 350,
        Mandar = 351,
        Mandingo = 352,
        Manipuri = 353,
        Manx = 354,
        Maori = 355,
        Mapuche = 356,
        Marathi = 357,
        Mari = 358,
        Marshallese = 359,
        Marwari = 360,
        Masai = 361,
        Mazanderani = 362,
        Medumba = 363,
        Mende = 364,
        Mentawai = 365,
        Meru = 366,
        Meta = 367,
        MexicanSpanish = 368,
        Michif = 369,
        MiddleDutch = 370,
        MiddleEnglish = 371,
        MiddleFrench = 372,
        MiddleHighGerman = 373,
        MiddleIrish = 374,
        Mikmaq = 375,
        MinNanChinese = 376,
        Minangkabau = 377,
        Mingrelian = 378,
        Mirandese = 379,
        Mizo = 380,
        ModernStandardArabic = 381,
        Mohawk = 382,
        Moksha = 383,
        Moldavian = 384,
        Mongo = 385,
        Mongolian = 386,
        Montenegrin = 387,
        MooseCree = 388,
        Morisyen = 389,
        MoroccanArabic = 390,
        Mossi = 391,
        Multiplelanguages = 392,
        Mundang = 393,
        Muscogee = 394,
        MuslimTat = 395,
        Myene = 396,
        NKo = 397,
        Nama = 398,
        NauruLanguage = 399,
        Navajo = 400,
        Ndonga = 401,
        Neapolitan = 402,
        Nepali = 403,
        Newari = 404,
        Ngambay = 405,
        Ngiemboon = 406,
        Ngomba = 407,
        Nheengatu = 408,
        Nias = 409,
        NigerianPidgin = 410,
        Niuean = 411,
        Nogai = 412,
        Nolinguisticcontent = 413,
        NorthNdebele = 414,
        NorthernEastCree = 415,
        NorthernFrisian = 416,
        NorthernHaida = 417,
        NorthernLuri = 418,
        NorthernSami = 419,
        NorthernSotho = 420,
        NorthernTutchone = 421,
        NorthwesternOjibwa = 422,
        Norwegian = 423,
        NorwegianBokmal = 424,
        NorwegianNynorsk = 425,
        Novial = 426,
        Nuer = 427,
        Nyamwezi = 428,
        Nyanja = 429,
        Nyankole = 430,
        NyasaTonga = 431,
        Nyoro = 432,
        Nzima = 433,
        Obolo = 434,
        Occitan = 435,
        Odia = 436,
        OjiCree = 437,
        Ojibwa = 438,
        Okanagan = 439,
        OldEnglish = 440,
        OldFrench = 441,
        OldHighGerman = 442,
        OldIrish = 443,
        OldNorse = 444,
        OldPersian = 445,
        OldProvencal = 446,
        Oromo = 447,
        Osage = 448,
        Ossetic = 449,
        OttomanTurkish = 450,
        Pahlavi = 451,
        PalatineGerman = 452,
        Palauan = 453,
        Pali = 454,
        Pampanga = 455,
        Pangasinan = 456,
        Papiamento = 457,
        Pashto = 458,
        PennsylvaniaGerman = 459,
        Persian = 460,
        Phoenician = 461,
        Picard = 462,
        Piedmontese = 463,
        Pijin = 464,
        PlainsCree = 465,
        Plautdietsch = 466,
        Pohnpeian = 467,
        Polish = 468,
        Pontic = 469,
        Portuguese = 470,
        Prussian = 471,
        Punjabi = 472,
        Quechua = 473,
        Rajasthani = 474,
        Rapanui = 475,
        Rarotongan = 476,
        Riffian = 477,
        Rohingya = 478,
        Romagnol = 479,
        Romanian = 480,
        Romansh = 481,
        Romany = 482,
        Rombo = 483,
        Rotuman = 484,
        Roviana = 485,
        Rundi = 486,
        Russian = 487,
        Rusyn = 488,
        Rwa = 489,
        Saamia = 490,
        Saho = 491,
        SamaritanAramaic = 492,
        Samburu = 493,
        SamiSkolt = 494,
        SamiSouthern = 495,
        Samoan = 496,
        Samogitian = 497,
        Sandawe = 498,
        Sango = 499,
        Sangu = 500,
        Sanskrit = 501,
        Santali = 502,
        Sardinian = 503,
        Sasak = 504,
        SassareseSardinian = 505,
        SaterlandFrisian = 506,
        Saurashtra = 507,
        Scots = 508,
        ScottishGaelic = 509,
        Selayar = 510,
        Selkup = 511,
        Sena = 512,
        Seneca = 513,
        Serbian = 514,
        SerboCroatian = 515,
        Serer = 516,
        Seri = 517,
        SeselwaCreoleFrench = 518,
        Shambala = 519,
        Shan = 520,
        Shona = 521,
        SichuanYi = 522,
        Sicilian = 523,
        Sidamo = 524,
        Siksika = 525,
        Silesian = 526,
        SimplifiedChinese = 527,
        Sindhi = 528,
        Sinhala = 529,
        Slave = 530,
        Slovak = 531,
        Slovenian = 532,
        Soga = 533,
        Sogdien = 534,
        Somali = 535,
        Soninke = 536,
        SouthNdebele = 537,
        SouthernAltai = 538,
        SouthernEastCree = 539,
        SouthernHaida = 540,
        SouthernKurdish = 541,
        SouthernLushootseed = 542,
        SouthernSotho = 543,
        SouthernTutchone = 544,
        Spanish = 545,
        SrananTongo = 546,
        StandardMoroccanTamazight = 547,
        StraitsSalish = 548,
        Sukuma = 549,
        Sumerian = 550,
        Sundanese = 551,
        Susu = 552,
        Swahili = 553,
        SwampyCree = 554,
        Swati = 555,
        Swedish = 556,
        SwissFrench = 557,
        SwissGerman = 558,
        SwissHighGerman = 559,
        Syriac = 560,
        Tachelhit = 561,
        Tagalog = 562,
        Tagish = 563,
        Tahitian = 564,
        Tahltan = 565,
        TaiDam = 566,
        Taita = 567,
        Tajik = 568,
        Talysh = 569,
        Tamashek = 570,
        Tamil = 571,
        Taroko = 572,
        Tasawaq = 573,
        Tatar = 574,
        Telugu = 575,
        Tereno = 576,
        Teso = 577,
        Tetum = 578,
        Thai = 579,
        Tibetan = 580,
        Tigre = 581,
        Tigrinya = 582,
        Timne = 583,
        Tiv = 584,
        Tlingit = 585,
        TokPisin = 586,
        TokelauLanguage = 587,
        TokiPona = 588,
        Tongan = 589,
        TornedalenFinnish = 590,
        Torwali = 591,
        TraditionalChinese = 592,
        Tsakhur = 593,
        Tsakonian = 594,
        Tsimshian = 595,
        Tsonga = 596,
        Tswana = 597,
        Tulu = 598,
        Tumbuka = 599,
        TunisianArabic = 600,
        Turkish = 601,
        Turkmen = 602,
        Turoyo = 603,
        TuvaluLanguage = 604,
        Tuvinian = 605,
        Twi = 606,
        Tyap = 607,
        Udmurt = 608,
        Ugaritic = 609,
        Uighur = 610,
        Ukrainian = 611,
        Umbundu = 612,
        Unknownlanguage = 613,
        UpperSorbian = 614,
        Urdu = 615,
        Uzbek = 616,
        Vai = 617,
        Venda = 618,
        Venetian = 619,
        Veps = 620,
        Vietnamese = 621,
        Volapuk = 622,
        Voro = 623,
        Votic = 624,
        Vunjo = 625,
        Walloon = 626,
        Walser = 627,
        Waray = 628,
        Warlpiri = 629,
        Washo = 630,
        Wayuu = 631,
        Welsh = 632,
        WestFlemish = 633,
        WesternBalochi = 634,
        WesternCanadianInuktitut = 635,
        WesternFrisian = 636,
        WesternMari = 637,
        WesternOjibwa = 638,
        WesternPanjabi = 639,
        Wolaytta = 640,
        Wolof = 641,
        WoodsCree = 642,
        WuChinese = 643,
        Xhosa = 644,
        XiangChinese = 645,
        Yakut = 646,
        Yangben = 647,
        Yao = 648,
        Yapese = 649,
        Yemba = 650,
        Yiddish = 651,
        Yoruba = 652,
        Zapotec = 653,
        Zarma = 654,
        Zaza = 655,
        Zeelandic = 656,
        Zenaga = 657,
        Zhuang = 658,
        ZoroastrianDari = 659,
        Zulu = 660,
        Zuni = 661,

        Azeri = Azerbaijani,
        ChineseCantonese = Cantonese,
        ChineseMandarin = Chinese,
        Hinglish = HindiLatin,
        KurdishCentral = CentralKurdish,
        KurdishSorani = CentralKurdish,
        Kyrgyz = Kirghiz,
        MandarinChinese = Chinese,
        MyanmarLanguage = Burmese,
        NajdiArabic = ArabicNajdi,
        Pushto = Pashto,
        SamiInari = InariSami,
        SamiLule = LuleSami,
        SamiNorthern = NorthernSami,
        SimplifiedMandarinChinese = SimplifiedChinese,
        SkoltSami = SamiSkolt,
        SouthernSami = SamiSouthern,
        TraditionalMandarinChinese = TraditionalChinese,
        UKEnglish = BritishEnglish,
        USEnglish = AmericanEnglish,
        Uyghur = Uighur,

        LastLanguage = Zuni
    };

    enum Country {
        AnyCountry = 0,
        Afghanistan = 1,
        Africa = 2,
        AlandIslands = 3,
        Albania = 4,
        Algeria = 5,
        AmericanSamoa = 6,
        Americas = 7,
        Andorra = 8,
        Angola = 9,
        Anguilla = 10,
        Antarctica = 11,
        AntiguaAndBarbuda = 12,
        AotearoaNewZealand = 13,
        Argentina = 14,
        Armenia = 15,
        Aruba = 16,
        AscensionIsland = 17,
        Asia = 18,
        Australasia = 19,
        Australia = 20,
        Austria = 21,
        Azerbaijan = 22,
        Bahamas = 23,
        Bahrain = 24,
        Bangladesh = 25,
        Barbados = 26,
        Belarus = 27,
        Belgium = 28,
        Belize = 29,
        Benin = 30,
        Bermuda = 31,
        Bhutan = 32,
        Bolivia = 33,
        Bosnia = 34,
        Botswana = 35,
        BouvetIsland = 36,
        Brazil = 37,
        BritishIndianOceanTerritory = 38,
        BritishVirginIslands = 39,
        Brunei = 40,
        Bulgaria = 41,
        BurkinaFaso = 42,
        Burundi = 43,
        CaboVerde = 44,
        Cambodia = 45,
        Cameroon = 46,
        Canada = 47,
        CanaryIslands = 48,
        Caribbean = 49,
        CaribbeanNetherlands = 50,
        CaymanIslands = 51,
        CentralAfricanRepublic = 52,
        CentralAmerica = 53,
        CentralAsia = 54,
        CeutaAndMelilla = 55,
        Chad = 56,
        Chile = 57,
        China = 58,
        ChristmasIsland = 59,
        ClippertonIsland = 60,
        CocosKeelingIslands = 61,
        Colombia = 62,
        Comoros = 63,
        CongoBrazzaville = 64,
        CongoDRC = 65,
        CookIslands = 66,
        CostaRica = 67,
        CotedIvoire = 68,
        Croatia = 69,
        Cuba = 70,
        Curacao = 71,
        Cyprus = 72,
        CzechRepublic = 73,
        Denmark = 74,
        DiegoGarcia = 75,
        Djibouti = 76,
        Dominica = 77,
        DominicanRepublic = 78,
        EastTimor = 79,
        EasternAfrica = 80,
        EasternAsia = 81,
        EasternEurope = 82,
        Ecuador = 83,
        Egypt = 84,
        ElSalvador = 85,
        EquatorialGuinea = 86,
        Eritrea = 87,
        Estonia = 88,
        Eswatini = 89,
        Ethiopia = 90,
        Europe = 91,
        EuropeanUnion = 92,
        Eurozone = 93,
        FalklandIslands = 94,
        FaroeIslands = 95,
        Fiji = 96,
        Finland = 97,
        France = 98,
        FrenchGuiana = 99,
        FrenchPolynesia = 100,
        FrenchSouthernTerritories = 101,
        Gabon = 102,
        Gambia = 103,
        Georgia = 104,
        Germany = 105,
        Ghana = 106,
        Gibraltar = 107,
        Greece = 108,
        Greenland = 109,
        Grenada = 110,
        Guadeloupe = 111,
        Guam = 112,
        Guatemala = 113,
        Guernsey = 114,
        Guinea = 115,
        GuineaBissau = 116,
        Guyana = 117,
        Haiti = 118,
        HeardAndMcDonaldIslands = 119,
        Honduras = 120,
        HongKong = 121,
        Hungary = 122,
        Iceland = 123,
        India = 124,
        Indonesia = 125,
        Iran = 126,
        Iraq = 127,
        Ireland = 128,
        IsleofMan = 129,
        Israel = 130,
        Italy = 131,
        Jamaica = 132,
        Japan = 133,
        Jersey = 134,
        Jordan = 135,
        Kazakhstan = 136,
        Kenya = 137,
        Kiribati = 138,
        Kosovo = 139,
        Kuwait = 140,
        Kyrgyzstan = 141,
        Laos = 142,
        LatinAmerica = 143,
        Latvia = 144,
        Lebanon = 145,
        Lesotho = 146,
        Liberia = 147,
        Libya = 148,
        Liechtenstein = 149,
        Lithuania = 150,
        Luxembourg = 151,
        Macao = 152,
        Madagascar = 153,
        Malawi = 154,
        Malaysia = 155,
        Maldives = 156,
        Mali = 157,
        Malta = 158,
        MarshallIslands = 159,
        Martinique = 160,
        Mauritania = 161,
        Mauritius = 162,
        Mayotte = 163,
        Melanesia = 164,
        Mexico = 165,
        Micronesia = 166,
        MicronesianRegion = 167,
        MiddleAfrica = 168,
        Moldova = 169,
        Monaco = 170,
        Mongolia = 171,
        Montenegro = 172,
        Montserrat = 173,
        Morocco = 174,
        Mozambique = 175,
        Myanmar = 176,
        Namibia = 177,
        Nauru = 178,
        Nepal = 179,
        Netherlands = 180,
        NewCaledonia = 181,
        Nicaragua = 182,
        Niger = 183,
        Nigeria = 184,
        Niue = 185,
        NorfolkIsland = 186,
        NorthAmerica = 187,
        NorthKorea = 188,
        NorthMacedonia = 189,
        NorthernAfrica = 190,
        NorthernAmerica = 191,
        NorthernEurope = 192,
        NorthernMarianaIslands = 193,
        Norway = 194,
        Oceania = 195,
        Oman = 196,
        OutlyingOceania = 197,
        Pakistan = 198,
        Palau = 199,
        Palestine = 200,
        Panama = 201,
        PapuaNewGuinea = 202,
        Paraguay = 203,
        Peru = 204,
        Philippines = 205,
        PitcairnIslands = 206,
        Poland = 207,
        Polynesia = 208,
        Portugal = 209,
        PseudoAccents = 210,
        PseudoBidi = 211,
        PuertoRico = 212,
        Qatar = 213,
        Reunion = 214,
        Romania = 215,
        Russia = 216,
        Rwanda = 217,
        Samoa = 218,
        SanMarino = 219,
        SaoTomeAndPrincipe = 220,
        Sark = 221,
        SaudiArabia = 222,
        Senegal = 223,
        Serbia = 224,
        Seychelles = 225,
        SierraLeone = 226,
        Singapore = 227,
        SintMaarten = 228,
        Slovakia = 229,
        Slovenia = 230,
        SolomonIslands = 231,
        Somalia = 232,
        SouthAfrica = 233,
        SouthAmerica = 234,
        SouthGeorgiaAndSouthSandwichIslands = 235,
        SouthKorea = 236,
        SouthSudan = 237,
        SoutheastAsia = 238,
        SouthernAfrica = 239,
        SouthernAsia = 240,
        SouthernEurope = 241,
        Spain = 242,
        SriLanka = 243,
        StBarthelemy = 244,
        StHelena = 245,
        StKittsAndNevis = 246,
        StLucia = 247,
        StMartin = 248,
        StPierreAndMiquelon = 249,
        StVincentAndGrenadines = 250,
        SubSaharanAfrica = 251,
        Sudan = 252,
        Suriname = 253,
        SvalbardAndJanMayen = 254,
        Sweden = 255,
        Switzerland = 256,
        Syria = 257,
        Taiwan = 258,
        Tajikistan = 259,
        Tanzania = 260,
        Thailand = 261,
        Togo = 262,
        Tokelau = 263,
        Tonga = 264,
        TrinidadAndTobago = 265,
        TristandaCunha = 266,
        Tunisia = 267,
        Turkey = 268,
        Turkmenistan = 269,
        TurksAndCaicosIslands = 270,
        Tuvalu = 271,
        UK = 272,
        UN = 273,
        US = 274,
        Uganda = 275,
        Ukraine = 276,
        UnitedArabEmirates = 277,
        UnitedStatesOutlyingIslands = 278,
        UnitedStatesVirginIslands = 279,
        UnknownRegion = 280,
        Uruguay = 281,
        Uzbekistan = 282,
        Vanuatu = 283,
        VaticanCity = 284,
        Venezuela = 285,
        Vietnam = 286,
        WallisAndFutuna = 287,
        WesternAfrica = 288,
        WesternAsia = 289,
        WesternEurope = 290,
        WesternSahara = 291,
        Yemen = 292,
        Zambia = 293,
        Zimbabwe = 294,
        world = 295,

        BosniaAndHerzegovina = Bosnia,
        CapeVerde = CaboVerde,
        CongoKinshasa = CongoDRC,
        CongoRepublic = CongoBrazzaville,
        Czechia = CzechRepublic,
        FalklandIslandsIslasMalvinas = FalklandIslands,
        HongKongSARChina = HongKong,
        IvoryCoast = CotedIvoire,
        MacaoSARChina = Macao,
        MyanmarBurma = Myanmar,
        NewZealand = AotearoaNewZealand,
        PalestinianTerritories = Palestine,
        Swaziland = Eswatini,
        TimorLeste = EastTimor,
        Turkiye = Turkey,
        UnitedKingdom = UK,
        UnitedNations = UN,
        UnitedStates = US,

        LastCountry = world
    };

    enum Script {
        AnyScript = 0,
        AdlamScript = 1,
        AfakaScript = 2,
        AhomScript = 3,
        AnatolianHieroglyphsScript = 4,
        ArabicScript = 5,
        ArmenianScript = 6,
        AvestanScript = 7,
        BalineseScript = 8,
        BamumScript = 9,
        BanglaScript = 10,
        BassaVahScript = 11,
        BatakScript = 12,
        BhaiksukiScript = 13,
        BlissymbolsScript = 14,
        BookPahlaviScript = 15,
        BopomofoScript = 16,
        BrahmiScript = 17,
        BrailleScript = 18,
        BugineseScript = 19,
        BuhidScript = 20,
        CarianScript = 21,
        CaucasianAlbanianScript = 22,
        ChakmaScript = 23,
        ChamScript = 24,
        CherokeeScript = 25,
        ChorasmianScript = 26,
        CirthScript = 27,
        CopticScript = 28,
        CypriotScript = 29,
        CyproMinoanScript = 30,
        CyrillicScript = 31,
        DeseretScript = 32,
        DevanagariScript = 33,
        DivesAkuruScript = 34,
        DograScript = 35,
        DuployanshorthandScript = 36,
        EasternSyriacScript = 37,
        EgyptiandemoticScript = 38,
        EgyptianhieraticScript = 39,
        EgyptianhieroglyphsScript = 40,
        ElbasanScript = 41,
        ElymaicScript = 42,
        EmojiScript = 43,
        EstrangeloSyriacScript = 44,
        EthiopicScript = 45,
        FrakturLatinScript = 46,
        FraserScript = 47,
        GaelicLatinScript = 48,
        GeorgianKhutsuriScript = 49,
        GeorgianScript = 50,
        GlagoliticScript = 51,
        GothicScript = 52,
        GranthaScript = 53,
        GreekScript = 54,
        GujaratiScript = 55,
        GunjalaGondiScript = 56,
        GurmukhiScript = 57,
        HanScript = 58,
        HangulScript = 59,
        HanifiRohingyaScript = 60,
        HanunooScript = 61,
        HanwithBopomofoScript = 62,
        HatranScript = 63,
        HebrewScript = 64,
        HiraganaScript = 65,
        ImperialAramaicScript = 66,
        IndusScript = 67,
        InheritedScript = 68,
        InscriptionalPahlaviScript = 69,
        InscriptionalParthianScript = 70,
        JamoScript = 71,
        JapaneseScript = 72,
        JapanesesyllabariesScript = 73,
        JavaneseScript = 74,
        JurchenScript = 75,
        KaithiScript = 76,
        KannadaScript = 77,
        KatakanaScript = 78,
        KawiScript = 79,
        KayahLiScript = 80,
        KharoshthiScript = 81,
        KhitansmallscriptScript = 82,
        KhmerScript = 83,
        KhojkiScript = 84,
        KhudawadiScript = 85,
        KoreanScript = 86,
        KpelleScript = 87,
        LannaScript = 88,
        LaoScript = 89,
        LatinScript = 90,
        LepchaScript = 91,
        LimbuScript = 92,
        LinearAScript = 93,
        LinearBScript = 94,
        LomaScript = 95,
        LycianScript = 96,
        LydianScript = 97,
        MahajaniScript = 98,
        MakasarScript = 99,
        MalayalamScript = 100,
        MandaeanScript = 101,
        ManichaeanScript = 102,
        MarchenScript = 103,
        MasaramGondiScript = 104,
        MathematicalNotationScript = 105,
        MayanhieroglyphsScript = 106,
        MedefaidrinScript = 107,
        MeiteiMayekScript = 108,
        MendeScript = 109,
        MeroiticCursiveScript = 110,
        MeroiticScript = 111,
        ModiScript = 112,
        MongolianScript = 113,
        MoonScript = 114,
        MroScript = 115,
        MultaniScript = 116,
        MyanmarScript = 117,
        NKoScript = 118,
        NabataeanScript = 119,
        NagMundariScript = 120,
        NandinagariScript = 121,
        NastaliqScript = 122,
        NaxiGebaScript = 123,
        NewTaiLueScript = 124,
        NewaScript = 125,
        NushuScript = 126,
        NyiakengPuachueHmongScript = 127,
        OdiaScript = 128,
        OghamScript = 129,
        OlChikiScript = 130,
        OldChurchSlavonicCyrillicScript = 131,
        OldHungarianScript = 132,
        OldItalicScript = 133,
        OldNorthArabianScript = 134,
        OldPermicScript = 135,
        OldPersianScript = 136,
        OldSogdianScript = 137,
        OldSouthArabianScript = 138,
        OldUyghurScript = 139,
        OrkhonScript = 140,
        OsageScript = 141,
        OsmanyaScript = 142,
        PahawhHmongScript = 143,
        PalmyreneScript = 144,
        PauCinHauScript = 145,
        PhagspaScript = 146,
        PhoenicianScript = 147,
        PollardPhoneticScript = 148,
        PsalterPahlaviScript = 149,
        RejangScript = 150,
        RongorongoScript = 151,
        RunicScript = 152,
        SACuneiformScript = 153,
        SamaritanScript = 154,
        SaratiScript = 155,
        SaurashtraScript = 156,
        SharadaScript = 157,
        ShavianScript = 158,
        SiddhamScript = 159,
        SignWritingScript = 160,
        SimplifiedHanScript = 161,
        SinhalaScript = 162,
        SogdianScript = 163,
        SoraSompengScript = 164,
        SoyomboScript = 165,
        SundaneseScript = 166,
        SylotiNagriScript = 167,
        SymbolsScript = 168,
        SyriacScript = 169,
        TagalogScript = 170,
        TagbanwaScript = 171,
        TaiLeScript = 172,
        TaiVietScript = 173,
        TakriScript = 174,
        TamilScript = 175,
        TangsaScript = 176,
        TangutScript = 177,
        TeluguScript = 178,
        TengwarScript = 179,
        ThaanaScript = 180,
        ThaiScript = 181,
        TibetanScript = 182,
        TifinaghScript = 183,
        TirhutaScript = 184,
        TotoScript = 185,
        TraditionalHanScript = 186,
        UCASScript = 187,
        UgariticScript = 188,
        UnwrittenScript = 189,
        VaiScript = 190,
        VarangKshitiScript = 191,
        VisibleSpeechScript = 192,
        VithkuqiScript = 193,
        WanchoScript = 194,
        WesternSyriacScript = 195,
        WoleaiScript = 196,
        YezidiScript = 197,
        YiScript = 198,
        ZanabazarSquareScript = 199,
        ZawgyiScript = 200,

        HanifiScript = HanifiRohingyaScript,
        PersoArabicScript = ArabicScript,
        SimplifiedScript = SimplifiedHanScript,
        SumeroAkkadianCuneiformScript = SACuneiformScript,
        TraditionalScript = TraditionalHanScript,
        UnifiedCanadianAboriginalSyllabicsScript = UCASScript,

        LastScript = ZawgyiScript
    };
    // GENERATED PART ENDS HERE

    enum MeasurementSystem { MetricSystem, ImperialSystem, UKSystem };

    enum FormatType { LongFormat, ShortFormat, NarrowFormat };
    enum NumberOption {
        OmitGroupSeparator = 0x01,
        RejectGroupSeparator = 0x02
    };
    Q_DECLARE_FLAGS(NumberOptions, NumberOption)

    QLocale();
    QLocale(const QString &name);
    QLocale(Language language, Country country = AnyCountry);
    QLocale(Language language, Script script, Country country);
    QLocale(const QLocale &other);

    QLocale &operator=(const QLocale &other);

    Language language() const;
    Script script() const;
    Country country() const;
    QString name() const;

    QByteArray bcp47Name() const;
    QString nativeLanguageName() const;
    QString nativeCountryName() const;

    short toShort(const QString &s, bool *ok = nullptr, int base = 0) const;
    ushort toUShort(const QString &s, bool *ok = nullptr, int base = 0) const;
    int toInt(const QString &s, bool *ok = nullptr, int base = 0) const;
    uint toUInt(const QString &s, bool *ok = nullptr, int base = 0) const;
    qlonglong toLongLong(const QString &s, bool *ok = nullptr, int base = 0) const;
    qulonglong toULongLong(const QString &s, bool *ok = nullptr, int base = 0) const;
    float toFloat(const QString &s, bool *ok = nullptr) const;
    double toDouble(const QString &s, bool *ok = nullptr) const;

    QString toString(qlonglong i) const;
    QString toString(qulonglong i) const;
    inline QString toString(short i) const;
    inline QString toString(ushort i) const;
    inline QString toString(int i) const;
    inline QString toString(uint i) const;
    QString toString(double i, char f = 'g', int prec = 6) const;
    inline QString toString(float i, char f = 'g', int prec = 6) const;
    QString toString(const QDate &date, const QString &formatStr) const;
    QString toString(const QDate &date, FormatType format = LongFormat) const;
    QString toString(const QTime &time, const QString &formatStr) const;
    QString toString(const QTime &time, FormatType format = LongFormat) const;
    QString toString(const QDateTime &dateTime, FormatType format = LongFormat) const;
    QString toString(const QDateTime &dateTime, const QString &format) const;

    QString dateFormat(FormatType format = LongFormat) const;
    QString timeFormat(FormatType format = LongFormat) const;
    QString dateTimeFormat(FormatType format = LongFormat) const;
#ifndef QT_NO_DATESTRING
    QDate toDate(const QString &string, FormatType = LongFormat) const;
    QTime toTime(const QString &string, FormatType = LongFormat) const;
    QDateTime toDateTime(const QString &string, FormatType format = LongFormat) const;
    QDate toDate(const QString &string, const QString &format) const;
    QTime toTime(const QString &string, const QString &format) const;
    QDateTime toDateTime(const QString &string, const QString &format) const;
#endif

    // ### Qt 5: We need to return QString from these function since
    //           unicode data contains several characters for these fields.
    QChar decimalPoint() const;
    QChar groupSeparator() const;
    QChar percent() const;
    QChar zeroDigit() const;
    QChar negativeSign() const;
    QChar positiveSign() const;
    QChar exponential() const;

    QString monthName(int, FormatType format = LongFormat) const;
    QString standaloneMonthName(int, FormatType format = LongFormat) const;
    QString dayName(int, FormatType format = LongFormat) const;
    QString standaloneDayName(int, FormatType format = LongFormat) const;

    Qt::DayOfWeek firstDayOfWeek() const;
    QList<Qt::DayOfWeek> weekdays() const;

    QString amText() const;
    QString pmText() const;

    MeasurementSystem measurementSystem() const;

    Qt::LayoutDirection textDirection() const;

    QString toUpper(const QString &str) const;
    QString toLower(const QString &str) const;

    QStringList uiLanguages() const;

    inline bool operator==(const QLocale &other) const;
    inline bool operator!=(const QLocale &other) const;

    static QString languageToString(Language language);
    static QString countryToString(Country country);
    static QString scriptToString(Script script);
    static void setDefault(const QLocale &locale);

    static QLocale c() { return QLocale(C); }
    static QLocale system();

    static QList<QLocale> matchingLocales(QLocale::Language language, QLocale::Script script, QLocale::Country country);
    static QList<Country> countriesForLanguage(Language lang);

    void setNumberOptions(NumberOptions options);
    NumberOptions numberOptions() const;

private:
    friend class QLocalePrivate;
    // ### We now use this field to pack an index into locale_data and NumberOptions.
    // ### Qt 5: change to a QLocaleData *d; uint numberOptions.
    struct Data {
        quint16 index;
        QLocale::NumberOptions numberOptions;
    };

    Data p;
    const QLocalePrivate *d() const;
};
Q_DECLARE_TYPEINFO(QLocale, Q_MOVABLE_TYPE);
Q_DECLARE_OPERATORS_FOR_FLAGS(QLocale::NumberOptions)

inline QString QLocale::toString(short i) const
    { return toString(qlonglong(i)); }
inline QString QLocale::toString(ushort i) const
    { return toString(qulonglong(i)); }
inline QString QLocale::toString(int i) const
    { return toString(qlonglong(i)); }
inline QString QLocale::toString(uint i) const
    { return toString(qulonglong(i)); }
inline QString QLocale::toString(float i, char f, int prec) const
    { return toString(double(i), f, prec); }
inline bool QLocale::operator==(const QLocale &other) const
    { return d() == other.d() && numberOptions() == other.numberOptions(); }
inline bool QLocale::operator!=(const QLocale &other) const
    { return d() != other.d() || numberOptions() != other.numberOptions(); }

#ifndef QT_NO_DATASTREAM
Q_CORE_EXPORT QDataStream &operator<<(QDataStream &, const QLocale &);
Q_CORE_EXPORT QDataStream &operator>>(QDataStream &, QLocale &);
#endif

QT_END_NAMESPACE

#endif // QLOCALE_H
